﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** AsymmetricCipher.cs:                                         ***//
//*** 	Class for handling and compressing data packets.           ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    01.06.2003: Daniel Fabian - To fit SharpPrivacy's needs   ***//
//***    31.10.2013  James R. Stembridge - To fit the needs of     ***//
//***                Hermes Crypto Library                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;
using Hermes.Crypto.OpenPGP;
using Hermes.Crypto.Cipher.Math;

namespace Hermes.Crypto.Cipher
{

    public abstract class AsymmetricCipher : object
    {

        public abstract BigInteger Decrypt(BigInteger[] biCipher, SecretKeyPacket spkKey, string strPassphrase);

        public abstract BigInteger[] Encrypt(BigInteger biPlain, PublicKeyPacket pkpKey);

        public abstract BigInteger[][] Generate(int nbits);

        public abstract BigInteger[] Sign(BigInteger biHash, SecretKeyPacket spkKey, string strPassphrase);

        public abstract bool Verify(BigInteger[] biSignature, BigInteger biHash, PublicKeyPacket pkpKey);

    }
}
