﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** Parts of this code rely on portions of the go-mono           ***//
//*** implemenation of the cipher. These parts are copyright of    ***//
//*** their respective owners:                                     ***//
//***   System.Security.Cryptography.RC2.cs                        ***//
//***     Authors:                                                 ***//
//***     Andrew Birkett (andy@nobugs.org)                         ***//
//***     Sebastien Pouliot (spouliot@motus.com)                   ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** CAST5.cs:                                                    ***//
//*** 	Class for encrypting and decrypting using the CAST5        ***//
//***   cipher.                                                    ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    01.06.2003: Daniel Fabian - To fit SharpPrivacy's needs   ***//
//***    31.10.2013  James R. Stembridge - To fit the needs of     ***//
//***                Hermes Crypto Library                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;

namespace Hermes.Crypto.Cipher
{

    public abstract class CAST5 : SymmetricAlgorithm
    {

        public static new CAST5 Create()
        {
            return (CAST5)Activator.CreateInstance(Type.GetType("Hermes.Crypto.Cipher.CAST5CryptoServiceProvider"), null);
        }

        protected int EffectiveKeySizeValue;

        public virtual int EffectiveKeySize
        {
            get
            {
                if (EffectiveKeySizeValue == 0)
                    return KeySizeValue;
                else
                    return EffectiveKeySizeValue;
            }
            set
            {
                var messageLoader = new Windows.ApplicationModel.Resources.ResourceLoader();
                if (!IsLegalKeySize(LegalKeySizesValue, value))
                    //throw new System.Security.Cryptography.CryptographicException("key size not supported by algorithm");
                    throw new Exception(messageLoader.GetString("Cast5KeySizeError"));
                //throw new Exception("CAST5: key size not supported by algorithm");
                EffectiveKeySizeValue = value;
            }
        }

        // Overridden, which makes me suspect it changes effective keysize too?
        public override int KeySize
        {
            get
            {
                return KeySizeValue;
            }
            set
            {
                KeySizeValue = value;
            }
        }

        public CAST5()
        {
            KeySizeValue = 128;
            BlockSizeValue = 64;
            FeedbackSizeValue = 64;

            //valid keysizes are 40 to 128 bit in 8 bit steps
            LegalKeySizesValue = new KeySizes[1];
            LegalKeySizesValue[0] = new KeySizes(40, 128, 8);

            LegalBlockSizesValue = new KeySizes[1];
            LegalBlockSizesValue[0] = new KeySizes(64, 64, 0);
        }


    }
}
