﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** CipherMode.cs:                                               ***//
//*** 	enumeration                                                ***//
//***                                                              ***//
//*** Original Author: Sebastien Pouliot (spouliot@motus.com)      ***//
//***          (C) 2002, 2003 Motus Technologies Inc.              ***//
//***          (http://www.motus.com)                              ***//
//***           http://www.go-mono.com.                            ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    01.06.2003: Daniel Fabian - To fit SharpPrivacy's needs   ***//
//***    31.10.2013  James R. Stembridge - To fir the needs of     ***//
//***                Hermes Crypto Library                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//

using System;
using Windows.Security.Cryptography;
using Windows.Storage.Streams;
using System.Runtime.InteropServices.WindowsRuntime;

namespace Hermes.Crypto.Cipher {

	public class KeyBuilder {
	
		//static private RandomNumberGenerator rng;
	
		static KeyBuilder () 
		{
			//rng = RandomNumberGenerator.Create ();
		}
	
		static public byte[] Key (int size) 
		{
			byte[] key = new byte [size];
			//rng.GetBytes (key);
            UInt32 targetLen = (UInt32)size;
            // Generate random data and copy it to a buffer.
            IBuffer buffer = CryptographicBuffer.GenerateRandom(targetLen);
            key = buffer.ToArray();
			return key;
		}
	
		static public byte[] IV (int size) 
		{
			byte[] iv = new byte [size];
			//rng.GetBytes (iv);
            UInt32 targetLen = (UInt32)size;
            // Generate random data and copy it to a buffer.
            IBuffer buffer = CryptographicBuffer.GenerateRandom(targetLen);
            iv = buffer.ToArray();
			return iv;
		}
	}
	
	// Process an array as a sequence of blocks
	public class BlockProcessor {
		private ICryptoTransform transform;
		private byte[] block;
		private int blockSize;	// in bytes (not in bits)
		private int blockCount;
	
		public BlockProcessor (ICryptoTransform transform) 
			: this (transform, transform.InputBlockSize) {} 
	
		// some Transforms (like HashAlgorithm descendant) return 1 for
		// block size (which isn't their real internal block size)
		public BlockProcessor (ICryptoTransform transform, int blockSize)
		{
			this.transform = transform;
			this.blockSize = blockSize;
			block = new byte [blockSize];
		}
	
		~BlockProcessor () {
			// zeroize our block (so we don't retain any information)
			Array.Clear (block, 0, blockSize);
		}
	
		public void Initialize () {
			Array.Clear (block, 0, blockSize);
			blockCount = 0;
		}
	
		public void Core(byte[] rgb) {
			Core (rgb, 0, rgb.Length);
		}
	
		public void Core(byte[] rgb, int ib, int cb) {
			// 1. fill the rest of the "block"
			int n = System.Math.Min (blockSize - blockCount, cb);
			Array.Copy (rgb, ib, block, blockCount, n); 
			blockCount += n;
	
			// 2. if block is full then transform it
			if (blockCount == blockSize) {
				transform.TransformBlock (block, 0, blockSize, ref block, 0);
	
				// 3. transform any other full block in specified buffer
				int b = (int) ((cb - n) / blockSize);
				for (int i=0; i < b; i++) {
					transform.TransformBlock (rgb, n, blockSize, ref block, 0);
					n += blockSize;
				}
	
				// 4. if data is still present fill the "block" with the remainder
				blockCount = cb - n;
				if (blockCount > 0)
					Array.Copy (rgb, n, block, 0, blockCount);
			}
		}
	
		public byte[] Final () 
		{
			return transform.TransformFinalBlock (block, 0, blockCount);
		}
	}
}
