﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** ICryptoTransform.cs:                                         ***//
//*** 	interface                                                  ***//
//***                                                              ***//
//*** Original Author: Matthew S. Ford                             ***//
//***                  (Matthew.S.Ford@Rose-Hulman.Edu)            ***//
//***                                                              ***//
//***           http://www.go-mono.com.                            ***//
//***          Copyright 2001 by Matthew S. Ford.                  ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    Version: 0.1.0 (initial release)                          ***//
//***    01.06.2003: Daniel Fabian - To fit SharpPrivacy's needs   ***//
//***    31.10.2013  James R. Stembridge - To fir the needs of     ***//
//***                Hermes Crypto Library                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;

namespace Hermes.Crypto.Cipher
{

    /// <summary>
    /// Crytographic functions that can process a stream of bytes implement this interface.
    /// This works by stringing together one or more ICryptoTransform classes with a stream.
    /// Data is passed from one to the next without the need of outside buffering/intervention.
    /// </summary>
    public interface ICryptoTransform : IDisposable
    {

        bool CanReuseTransform { get; }

        /// <summary>
        /// Whether the function can transform multiple blocks at a time.
        /// </summary>
        bool CanTransformMultipleBlocks { get; }

        /// <summary>
        /// Size of input blocks for the function in bytes.
        /// </summary>
        int InputBlockSize { get; }

        /// <summary>
        /// Size of the output blocks of the function in bytes.
        /// </summary>
        int OutputBlockSize { get; }

        /// <summary>
        /// FIXME: Process some data.  A block at a time?  Less than a block at a time?
        /// </summary>
        int TransformBlock(byte[] inputBuffer, int inputOffset, int inputCount, ref byte[] outputBuffer, int outputOffset);

        /// <summary>
        /// Processes the final part of the data.  Also finalizes the function if needed.
        /// </summary>
        byte[] TransformFinalBlock(byte[] inputBuffer, int inputOffset, int inputCount);
    }

}

