﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** Parts of this code rely on portions of the go-mono           ***//
//*** implemenation of the cipher. These parts are copyright of    ***//
//*** their respective owners:                                     ***//
//***   System.Security.Cryptography.RC2.cs                        ***//
//***     Authors:                                                 ***//
//***     Dan Lewis (dihlewis@yahoo.co.uk)                         ***//
//***     Andrew Birkett (andy@nobugs.org)                         ***//
//***     (C) 2002                                                 ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Rijndael.cs                                                  ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    01.06.2003: Daniel Fabian - To fit SharpPrivacy's needs   ***//
//***    31.10.2013  James R. Stembridge - To fit the needs of     ***//
//***                Hermes Crypto Library                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;

namespace Hermes.Crypto.Cipher
{

    public abstract class Rijndael : SymmetricAlgorithm
    {

        public static new Rijndael Create()
        {
            return (Rijndael)Activator.CreateInstance(Type.GetType("Hermes.Crypto.Cipher.RijndaelManaged"), null);
        }

        public Rijndael()
        {
            KeySizeValue = 256;
            BlockSizeValue = 128;
            FeedbackSizeValue = 128;

            LegalKeySizesValue = new KeySizes[1];
            LegalKeySizesValue[0] = new KeySizes(128, 256, 64);

            LegalBlockSizesValue = new KeySizes[1];
            LegalBlockSizesValue[0] = new KeySizes(128, 256, 64);
        }
    }
}
