﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** EncryptionHelper.cs:                                         ***//
//*** 	This class implements helper functions for encrypting,     ***//
//***	decrypting, signing, verifying, etc.                       ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    Version: 0.1.0 (initial release)                          ***//
//***    01.06.2003: Daniel Fabian - Added header for the first    ***//
//***                beta release.                                 ***//
//***    31.10.2013  James R. Stembridge - This was an extensive   ***//
//***                rewrite of the module.  Consilidated several  ***//
//***                encrypt and decrypt functions into one each,  ***//
//***                removed code that prevented this module from  ***//
//***                working properly in the WinRT environment,    ***//
//***                removed code that required direct interaction ***//
//***                with the user, added call to allow a calling  ***//
//***                routine to retrieve the appropriate data, and ***//
//***                changed the namespace to conform with my      ***//
//***                project's needs.                              ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hermes.Crypto.Cipher;
using Hermes.Crypto.OpenPGP;
using Hermes.Crypto.OpenPGP.Messages;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Storage.Streams;
using Windows.Security.Cryptography.Core;

namespace Hermes.Crypto
{
    public class EncryptionHelper : object
    {
        private byte[] binaryReturn;
        private string _encryptedString;

        private string strSigDisplay = string.Empty;
        private string strDisplay = string.Empty;
        private byte[] _decryptedBytes;
        private byte[] mdcHash;
        private byte[] mdcRandom;
        private byte[] dataToHash;

        private static string signatureHeader;
        private static string signatureArmored;
        private static string signatureMessage;

        public string SignatureHeader
        {
            get
            {
                return signatureHeader;
            }
            set
            {
                signatureHeader = value; ;
            }
        }

        public string SignatureArmored
        {
            get
            {
                return signatureArmored;
            }
            set
            {
                signatureArmored = value; ;
            }
        }

        public string SignatureMessage
        {
            get
            {
                return signatureMessage;
            }
            set
            {
                signatureMessage = value; ;
            }
        }

        public string EncryptedString
        {
            get
            {
                return _encryptedString;
            }
            //set
            //{
            //    _encryptedString = value;
            //}
        }

        public byte[] DecryptedBytes
        {
            get
            {
                return _decryptedBytes;
            }
            //set
            //{
            //    _decryptedBytes = value;
            //}
        }

        public byte[] BinaryReturn
        {
            get
            {
                return binaryReturn;
            }
            //set
            //{
            //    binaryReturn = value;
            //}
        }

        public string DecryptedString
        {
            get
            {
                return strDisplay;
            }
            //set
            //{
            //    strDisplay = value;
            //}
        }

        public string SignatureString
        {
            get
            {
                return strSigDisplay;
            }
            //set
            //{
            //    strSigDisplay = value;
            //}
        }

        /// <summary>
        /// Encrypt - routine to facilitate encrypting a
        /// literal message.
        /// </summary>
        /// <param name="litMessage">The literal message to be encrypted. 
        /// Must be Literal Message format.</param>
        /// <param name="myPassPhrase">The user's pass phrase. Used for signing 
        /// the message.</param>
        /// <param name="transSecKey">The user's transportable secret key. Used
        /// for signing the message</param>
        /// <param name="transPublicKeys">A list of transportable puclic keys. 
        /// Used for encrypting the message</param>
        /// <param name="MDC">A bool indicator the specifies that all of the 
        /// recipient's keys qualify for MDC protected packets or not.</param>
        /// <param name="signMessage">A bool indicator the specifies the 
        /// message should be signed.</param>
        /// <param name="encryptMessage">A bool indicator the specifies the 
        /// message should be encrypted.</param>
        /// <returns>Nothing, you must query the module for the data you want.</returns>
        public void Encrypt(LiteralMessage litMessage, string myPassPhrase, TransportableSecretKey transSecKey, List<TransportablePublicKey> selectedPublicKeys, bool MDC, bool signMessage, bool encryptMessage)
        {
            string errorMessage = string.Empty;
            bool haveError = false;
            // Do the signing and encrypting 
            Hermes.Crypto.OpenPGP.Messages.Message encryptionMessage = litMessage;
            //Do signing stuff here!!!!!!!!!!!!!!!!!!!!!!
            if (signMessage)
            {
                // Sign and Ecrypt
                SignedMessage signedMessage = new SignedMessage();
                signedMessage.MessageSigned = litMessage;
                SignaturePacket sigPacket = new SignaturePacket();
                sigPacket.Version = SignaturePacketVersionNumbers.v3;
                // select the correct private key here
                SecretKeyPacket secKeyPacket = transSecKey.FindKey(AsymActions.Sign);
                sigPacket.KeyID = secKeyPacket.PublicKey.KeyID;
                sigPacket.TimeCreated = DateTime.Now;
                sigPacket.SignatureAlgorithm = secKeyPacket.PublicKey.Algorithm;
                sigPacket.SignatureType = SignatureTypes.TextSignature;
                sigPacket.HashAlgorithm = HashAlgorithms.SHA1;
                try
                {
                    sigPacket.Sign(litMessage.Binary, secKeyPacket, myPassPhrase);
                }
                catch (Exception ee)
                {
                    errorMessage = ee.Message;
                    //if (ee.Message == "Invalid Passphrase!")
                    myPassPhrase = string.Empty;
                    haveError = true;
                }
                if (haveError)
                {
                    throw new Exception(errorMessage);
                }
                signedMessage.Signature = sigPacket;
                encryptionMessage = signedMessage;
            }

            CompressedMessage compressedMessage = new CompressedMessage();
            compressedMessage.Compress(encryptionMessage);
            if (encryptMessage)
            {
                SymAlgorithms symAlgo = GetSymAlgorithmPreferences(selectedPublicKeys);
                // If all the keys support MDC (we checked this earlier when we built the list),
                // we need to use a Sym Enc and MDC protected packet. Otherwise we need to use 
                // the old Sym Enc data packet. We create both to facilitate downstream processing.
                SymEncryptedIntegrityProtectedDataPacket seipdpEncrypted = new SymEncryptedIntegrityProtectedDataPacket();
                SymmetricallyEncryptedDataPacket sedpEncrypted = new SymmetricallyEncryptedDataPacket();
                SymmetricAlgorithm symAlgoEncypt = CipherHelper.CreateSymAlgorithm(symAlgo);
                if (MDC)
                {
                    symAlgoEncypt.IntegrityProtected = true;
                }
                else
                {
                    symAlgoEncypt.IntegrityProtected = false;
                }
                symAlgoEncypt.Mode = CipherMode.OpenPGP_CFB;
                symAlgoEncypt.GenerateKey();
                byte[] bKey = symAlgoEncypt.Key;
                ESKSequence esksKeys = new ESKSequence();
                try
                {
                    esksKeys = CreateESKSequence(selectedPublicKeys, AsymActions.Encrypt, symAlgo, bKey);
                }
                catch (Exception ee)
                {
                    haveError = true;
                    errorMessage = ee.Message;
                }
                if (haveError)
                {
                    throw new Exception(errorMessage);
                }
                ICryptoTransform ictEncryptor = symAlgoEncypt.CreateEncryptor();
                byte[] binaryMessage = compressedMessage.GetEncoded();
                int extra = 0;
                if (MDC)
                    extra = 22;
                byte[] binaryOutput = new byte[binaryMessage.Length + extra];
                ictEncryptor.TransformBlock(binaryMessage, 0, binaryMessage.Length, ref binaryOutput, 0);
                bKey.Initialize();
                int x = 0;
                bool KeepLooking = true;
                for (x = binaryOutput.Length; KeepLooking; x--)
                {
                    if (binaryOutput[x - 1] != 0)
                        KeepLooking = false;
                    if (x < 10)
                        KeepLooking = false;
                }
                x++;
                int iOutLength = (symAlgoEncypt.BlockSize >> 3) + 2 + binaryMessage.Length;
                if (MDC)
                {
                    iOutLength = x;
                    seipdpEncrypted.Body = new byte[iOutLength + 1];
                    seipdpEncrypted.Body[0] = 0x01;
                    Array.Copy(binaryOutput, 0, seipdpEncrypted.Body, 1, iOutLength);
                    Array.Copy(binaryOutput, 0, seipdpEncrypted.Body, 1, x);
                }
                else
                {
                    //iOutLength = x;
                    sedpEncrypted.Body = new byte[iOutLength];
                    Array.Copy(binaryOutput, 0, sedpEncrypted.Body, 0, iOutLength);
                }
                byte[] binaryESK = esksKeys.GetEncoded();
                if (MDC)
                {
                    byte[] bEncrypted = seipdpEncrypted.Generate();
                    binaryReturn = new byte[binaryESK.Length + bEncrypted.Length];
                    binaryESK.CopyTo(binaryReturn, 0);
                    bEncrypted.CopyTo(binaryReturn, binaryESK.Length);
                }
                else
                {
                    byte[] bEncrypted = sedpEncrypted.Generate();
                    binaryReturn = new byte[binaryESK.Length + bEncrypted.Length];
                    binaryESK.CopyTo(binaryReturn, 0);
                    bEncrypted.CopyTo(binaryReturn, binaryESK.Length);
                }
            }
            else
            {
                binaryReturn = compressedMessage.GetEncoded();
            }
            if (litMessage.DataFormat == DataFormatTypes.Text)
            {
                _encryptedString = Radix64.Encode(binaryReturn, true);
                _encryptedString = Armor.WrapMessage(_encryptedString);
            }
            else
            {
                _encryptedString = string.Empty;

            }


        }

        /// <summary>
        /// Decrypt - routine to facilitate decrypting data. </summary>
        /// <param name="binaryData"> - A byte array containing the data to be
        /// decrypted.</param>
        /// <param name="publicKeyRing"> - The public key ring to be used for  
        /// this operation.</param>
        /// <param name="secretKeyRing"> - The secret key ring to be used for  
        /// this operation.</param>
        /// <param name="myPassPhrase"> - The user's pass phrase. Used for decrypting 
        /// the message.</param>
        /// <param name="myPrimaryKeylKeyID"> - The ulong number of this user's primary key.
        ///  Used as the default key for decrypting the message.</param>
        /// <returns>Returns - Nothing, you must query the module for the data you want.</returns>
        /// 
        public void Decrypt(byte[] binaryData, PublicKeyRing publicKeyRing, SecretKeyRing secretKeyRing, string myPassPhrase, ulong myPrimaryKeylKeyID)
        {
            bool messageSignedOnly = false;

            var messageLoader = new Windows.ApplicationModel.Resources.ResourceLoader();

            string mystr = System.Text.Encoding.UTF8.GetString(binaryData, 0, binaryData.Length);
            ArmorTypes armorType = new ArmorTypes();
            string strRest = "";
            strDisplay = "";
            string errorMessage = string.Empty;
            bool haveError = false;

            string strRadix64 = Armor.RemoveArmor(mystr, ref armorType, ref strRest);
            if (strRadix64.Length > 0)
                binaryData = Radix64.Decode(strRadix64);
            Message msgContent = null;
            if (armorType == ArmorTypes.OpenPGPSignedMessage)
            {
                messageSignedOnly = true;
                string strSignature = "";
                string strSignedMessage = Armor.RemoveClearSignatureArmor(mystr, ref armorType, ref strSignature);
                strSignedMessage = Radix64.DashUnescape(strSignedMessage);
                strSignedMessage = Radix64.TrimMessage(strSignedMessage);
                SignedMessage signedMessage = new SignedMessage();
                Packet[] parsedPackets = Packet.ParsePackets(strSignature);
                if (!(parsedPackets[0] is SignaturePacket))
                {
                    //MessageBox.Show("Not a valid cleartext signature!");
                    //MessageDialog dialog = new MessageDialog("Not a valid cleartext signature!", "Error");
                    throw new Exception(messageLoader.GetString("InvalidClearTextSig"));
                }
                signedMessage.Signature = (SignaturePacket)parsedPackets[0];
                LiteralMessage litMessage = new LiteralMessage(DataFormatTypes.Text);
                litMessage.Text = strSignedMessage;
                signedMessage.MessageSigned = litMessage;
                msgContent = signedMessage;
            }
            else
            {
                // let us see what kind of message this is
                EncryptedMessage encryptedMessage = new EncryptedMessage();
                try
                {
                    Packet[] parsedPackets = Packet.ParsePackets(binaryData);
                    encryptedMessage.ParseMessage(parsedPackets);
                    if (encryptedMessage.SymmetricallyEncrypted)
                    {
                        msgContent = encryptedMessage.Decrypt(myPassPhrase);
                    }
                    else
                    {
                        msgContent = encryptedMessage.Decrypt(secretKeyRing, myPassPhrase, myPrimaryKeylKeyID);
                    }
                    while ((!(msgContent is LiteralMessage)) && (!(msgContent is SignedMessage)))
                    {
                        if (msgContent is CompressedMessage)
                        {
                            msgContent = ((CompressedMessage)msgContent).Uncompress();
                        }
                        else
                        {
                            //MessageBox.Show("This is not a valid OpenPGP message!");
                            //MessageDialog dialog = new MessageDialog("This is not a valid OpenPGP message!", "Error");
                            throw new Exception(messageLoader.GetString("InvalidOpenPGPMessage"));
                        }
                    }
                }
                catch (Exception ee)
                {
                    //MessageBox.Show("There was an error decrypting your message: " + ee.Message);
                    errorMessage = ee.Message;
                    //if (ee.Message == "Invalid Passphrase!")
                    myPassPhrase = string.Empty;
                    haveError = true;
                }
                if (haveError)
                {
                    throw new Exception(errorMessage);
                }
                mdcHash = encryptedMessage.MDCHash;
                mdcRandom = encryptedMessage.MDCRandom;
                dataToHash = encryptedMessage.MDCData;
            }
            LiteralMessage litMsgContent = new LiteralMessage();
            strDisplay = string.Empty;
            strSigDisplay = string.Empty;
            if (msgContent is SignedMessage)
            {
                SignedMessage signedMsgContent = (SignedMessage)msgContent;

                litMsgContent = signedMsgContent.MessageSigned;

                if (mdcHash != null)
                    //strSigDisplay += "*** OpenPGP Encrypted Message - Signed, MDC protected ***\r\n";
                    strSigDisplay += messageLoader.GetString("OpenPGPEncryptedSignedMDC") + "\r\n";
                else
                    //strSigDisplay += "*** OpenPGP Encrypted Message - Signed ***\r\n";
                    strSigDisplay += messageLoader.GetString("OpenPGPEncryptedSigned") + "\r\n";
                if (messageSignedOnly)
                    strSigDisplay = messageLoader.GetString("OpenPGPSignedNotEncrypted") + "\r\n";
                //strSigDisplay = "*** OpenPGP Signed Message - Not Encrypted ***\r\n";
                //strSigDisplay += "*** Signature Status: " + signedMsgContent.Verify(publicKeyRing) + " ***\r\n";
                strSigDisplay += messageLoader.GetString("SigStatus") + signedMsgContent.Verify(publicKeyRing) + " ***\r\n";
                //strSigDisplay += "*** Signing Key: " + signedMsgContent.Signature.KeyID.ToString("X") + " ***\r\n";
                strSigDisplay += messageLoader.GetString("SigningKey") + signedMsgContent.Signature.KeyID.ToString("X") + " ***\r\n";
                //string strWork = "Unknown signer. Key not found in Public Key Ring.";
                string strWork = messageLoader.GetString("UnknownSigner");
                IEnumerator<TransportablePublicKey> ieKeys = publicKeyRing.PublicKeys.GetEnumerator();
                while (ieKeys.MoveNext())
                {
                    if (!(ieKeys.Current is TransportablePublicKey))
                    {
                        continue;
                    }
                    TransportablePublicKey transPubKey = (TransportablePublicKey)ieKeys.Current;
                    string strSize = transPubKey.PrimaryKey.KeyMaterial[0].bitCount().ToString();
                    string entryItem = string.Empty;
                    if (signedMsgContent.Signature.KeyID.ToString("x") == transPubKey.PrimaryKey.KeyID.ToString("x"))
                    {
                        strWork = transPubKey.PrimaryUserID;
                    }
                    else if (transPubKey.SubKeys.Count > 0)
                    {
                        if (transPubKey.SubKeys[0].Subkey.KeyID.ToString("x") == signedMsgContent.Signature.KeyID.ToString("x"))
                        {
                            strWork = transPubKey.PrimaryUserID;
                        }
                    }
                }
                //strSigDisplay += "*** Signer's Name: " + strWork + " ***\r\n";
                strSigDisplay += messageLoader.GetString("SignersName") + strWork + " ***\r\n";
                //strSigDisplay += "*** Signing Date/Time: " + signedMsgContent.Signature.TimeCreated.ToString() + "(GMT) ***\r\n";
                strSigDisplay += messageLoader.GetString("SigningDate") + signedMsgContent.Signature.TimeCreated.ToString()  + " ***\r\n";
            }
            else if (msgContent is LiteralMessage)
            {
                litMsgContent = (LiteralMessage)msgContent;
                if (mdcHash != null)
                    //strSigDisplay += "*** OpenPGP Encrypted Message - Not Signed, MDC protected ***\r\n";
                    strSigDisplay += messageLoader.GetString("OpenPGPEncryptedMDC") + "\r\n";
                else
                    //strSigDisplay += "*** OpenPGP Encrypted Message - Not Signed ***\r\n";
                    strSigDisplay += messageLoader.GetString("OpenPGPEncrypted") + "\r\n";
            }
            else
            {
                throw new Exception(messageLoader.GetString("NotEncryptedOrSigned"));
            }
            try
            {
                if (mdcHash != null)
                {
                    byte[] wkData = dataToHash;
                    byte[] hData = new byte[mdcRandom.Length + wkData.Length + 2];
                    Array.Copy(mdcRandom, hData, mdcRandom.Length);
                    Array.Copy(wkData, 0, hData, mdcRandom.Length, wkData.Length);
                    byte[] prefix = new byte[2];
                    prefix[0] = 0xD3;
                    prefix[1] = 0x14;
                    Array.Copy(prefix, 0, hData, mdcRandom.Length + wkData.Length, prefix.Length);
                    string hashAlg = HashAlgorithmNames.Sha1;
                    // Create a HashAlgorithmProvider object.
                    HashAlgorithmProvider objAlgProv = HashAlgorithmProvider.OpenAlgorithm(hashAlg);
                    IBuffer buffMsg = hData.AsBuffer();
                    // Hash the message.
                    IBuffer buffHash = objAlgProv.HashData(buffMsg);
                    // Verify that the hash length equals the length specified for the algorithm.
                    if (buffHash.Length != objAlgProv.HashLength)
                    {
                        //strSigDisplay += "*** WARNING: MDC Packet Hash Length Error. This message may have been tampered with. ***\r\n";
                        strSigDisplay += messageLoader.GetString("MDCLengthError") + "\r\n";
                    }
                    // Convert the IBuffer (hash) to a Byte Array
                    byte[] myHash = buffHash.ToArray();
                    bool hashError = false;
                    for (int x = 0; x < myHash.Length; x++)
                    {
                        if (myHash[x] != mdcHash[x])
                        {
                            hashError = true;
                            break;
                        }
                    }
                    if (hashError)
                        strSigDisplay += messageLoader.GetString("MDCHashError") + "\r\n";
                    //strSigDisplay += "*** WARNING: MDC Packet Hash does not match. This message may have been tampered with. ***\r\n";
                }
            }
            catch
            {
                strSigDisplay += messageLoader.GetString("MDCPacketNotFound") + "\r\n";
                //strSigDisplay += "*** ERROR: MDC Packet not found. This message may have been tampered with. ***\r\n";
            }
            strSigDisplay += "\r\n";
            _decryptedBytes = litMsgContent.Binary;
            if (litMsgContent.DataFormat == DataFormatTypes.Text)
            {
                strDisplay += litMsgContent.Text;
            }
            else
            {
                strDisplay = string.Empty;
            }
        }

        /// <summary>
        /// SignOnly - routine used when the user wants to sign, but not
        /// encrypt a text message.
        /// </summary>
        /// <param name="myStr">The message text string to be signed.
        /// </param>
        /// <param name="myPassPhrase">The user's pass phrase. Used for decrypting 
        /// the message.</param>
        /// <param name="transSecKey">The user's transportable secret key. Used
        /// for signing the message</param>
        /// <returns>Nothing, you must query the module for the data you want.</returns>
        public void signOnly(string mystr, string myPassPhrase, TransportableSecretKey transSecKey)
        {

            var messageLoader = new Windows.ApplicationModel.Resources.ResourceLoader();
            bool haveError = false;
            string errorMessage = string.Empty;

            mystr = Radix64.TrimMessage(mystr);
            SignaturePacket sigPktSign = new SignaturePacket();
            SecretKeyPacket secKeyPkt = transSecKey.FindKey(AsymActions.Sign);
            sigPktSign.Format = PacketFormats.New;
            sigPktSign.HashAlgorithm = HashAlgorithms.SHA1;
            SignatureSubPacket sigPktCreator = new SignatureSubPacket();
            sigPktCreator.Type = SignatureSubPacketTypes.IssuerKeyID;
            sigPktCreator.KeyID = secKeyPkt.PublicKey.KeyID;
            SignatureSubPacket sigPktCreationTime = new SignatureSubPacket();
            sigPktCreationTime.Type = SignatureSubPacketTypes.SignatureCreationTime;
            sigPktCreationTime.TimeCreated = DateTime.Now;
            sigPktSign.HashedSubPackets = new SignatureSubPacket[2];
            sigPktSign.HashedSubPackets[0] = sigPktCreator;
            sigPktSign.HashedSubPackets[1] = sigPktCreationTime;
            sigPktSign.SignatureAlgorithm = secKeyPkt.PublicKey.Algorithm;
            sigPktSign.SignatureType = SignatureTypes.TextSignature;
            sigPktSign.Version = SignaturePacketVersionNumbers.v4;
            byte[] binaryMessage = System.Text.Encoding.UTF8.GetBytes(mystr);
            try
            {
                sigPktSign.Sign(binaryMessage, secKeyPkt, myPassPhrase);
            }
            catch (Exception e)
            {
                errorMessage = e.Message;
                //if (ee.Message == "Invalid Passphrase!")
                myPassPhrase = string.Empty;
                haveError = true;
            }
            if (haveError)
            {
                //MessageDialog dialog = new MessageDialog(errorMessage, "Error");
                throw new Exception(errorMessage);
                //MessageDialog dialog = new MessageDialog(errorMessage, messageLoader.GetString("Error"));
                //await dialog.ShowAsync();
                //haveError = false;
                //errorMessage = string.Empty;
                //throw new Exception(messageLoader.GetString("AbortEncrypt"));
                //throw new Exception("Aborting encryption process.");
            }
            byte[] binarySignature = sigPktSign.Generate();
            string strSignature = Radix64.Encode(binarySignature, true);
            _encryptedString = Armor.WrapCleartextSignature(mystr, strSignature);
            this.SignatureArmored = Armor.SignatureArmored;
            this.SignatureHeader = Armor.SignatureHeader;
            this.SignatureMessage = Armor.SignatureMessage;

        }



        private SymAlgorithms GetSymAlgorithmPreferences(List<TransportablePublicKey> listPublicKeys)
        {
            bool bCAST5 = true;
            bool bAES256 = true;
            bool bAES192 = true;
            bool bAES128 = true;
            IEnumerator<TransportablePublicKey> ieKeys = listPublicKeys.GetEnumerator();
            while (ieKeys.MoveNext())
            {
                if (!(ieKeys.Current is TransportablePublicKey))
                    continue;
                TransportablePublicKey transPubKey = (TransportablePublicKey)ieKeys.Current;
                ulong lKeyID = transPubKey.PrimaryKey.KeyID;
                IEnumerator<CertifiedUserID> ieCerts = transPubKey.Certifications.GetEnumerator();
                while (ieCerts.MoveNext())
                {
                    if (!(ieCerts.Current is CertifiedUserID))
                        continue;
                    CertifiedUserID certUserID = (CertifiedUserID)ieCerts.Current;
                    IEnumerator<SignaturePacket> ieSigs = certUserID.Certificates.GetEnumerator();
                    while (ieSigs.MoveNext())
                    {
                        if (!(ieSigs.Current is SignaturePacket))
                            continue;
                        SignaturePacket sigPktSignature = (SignaturePacket)ieSigs.Current;
                        if ((sigPktSignature.Version == SignaturePacketVersionNumbers.v4) && (sigPktSignature.KeyID == lKeyID))
                        {
                            try
                            {
                                bool bTmpCAST5 = false;
                                bool bTmpAES256 = false;
                                bool bTmpAES192 = false;
                                bool bTmpAES128 = false;
                                SymAlgorithms[] symAlgoThisKey = sigPktSignature.FindPreferedSymAlgorithms();
                                for (int i = 0; i < symAlgoThisKey.Length; i++)
                                {
                                    if (symAlgoThisKey[i] == SymAlgorithms.AES128)
                                        bTmpAES128 = true;
                                    else if (symAlgoThisKey[i] == SymAlgorithms.AES192)
                                        bTmpAES192 = true;
                                    else if (symAlgoThisKey[i] == SymAlgorithms.AES256)
                                        bTmpAES256 = true;
                                    else if (symAlgoThisKey[i] == SymAlgorithms.CAST5)
                                        bTmpCAST5 = true;
                                }
                                if (!bTmpCAST5)
                                    bCAST5 = false;
                                if (!bTmpAES256)
                                    bAES256 = false;
                                if (!bTmpAES192)
                                    bAES192 = false;
                                if (!bTmpAES128)
                                    bAES128 = false;
                            }
                            catch (InvalidOperationException) { }
                        }
                    }
                }
            }
            if (bAES256)
                return SymAlgorithms.AES256;
            if (bAES192)
                return SymAlgorithms.AES192;
            if (bAES128)
                return SymAlgorithms.AES128;
            if (bCAST5)
                return SymAlgorithms.CAST5;
            return SymAlgorithms.Triple_DES;
        }

        private ESKSequence CreateESKSequence(List<TransportablePublicKey> listKeys, AsymActions aaAction, SymAlgorithms symAlgo, byte[] bSymKey)
        {
            var messageLoader = new Windows.ApplicationModel.Resources.ResourceLoader();
            IEnumerator<TransportablePublicKey> ieKeys = listKeys.GetEnumerator();
            ESKSequence encSessionKeys = new ESKSequence();
            while (ieKeys.MoveNext())
            {
                TransportablePublicKey transPubKey = (TransportablePublicKey)ieKeys.Current;
                PublicKeyPacket pubKeyPkt = transPubKey.FindKey(aaAction);
                if (pubKeyPkt == null)
                    //throw new Exception("Could not find fitting subkey for the selected action. Concerned Key: " + transPubKey.PrimaryUserID);
                    throw new Exception(messageLoader.GetString("NoFittingSubKey") + transPubKey.PrimaryUserID);
                AsymSessionKeyPacket sessionKeyPkt = new AsymSessionKeyPacket();
                sessionKeyPkt.KeyID = pubKeyPkt.KeyID;
                //sessionKeyPkt.KeyID = transPubKey.PrimaryKey.KeyID;
                sessionKeyPkt.PublicAlgorithm = pubKeyPkt.Algorithm;
                sessionKeyPkt.SymmetricAlgorithm = symAlgo;
                sessionKeyPkt.SessionKey = bSymKey;
                sessionKeyPkt.EncryptSessionKey(pubKeyPkt);
                encSessionKeys.AddAsymSessionKey(sessionKeyPkt);
            }
            return encSessionKeys;
        }

    }
}
