//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright  2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Armor.cs:                                                    ***//
//*** 	Class for handling the openPGP ascii armor.                ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    Version: 0.1.0 (initial release)                          ***//
//***    11.03.2003: Daniel Fabian - Created this file.            ***//
//***    01.06.2003: Daniel Fabian - Added header for the first    ***//
//***                beta release.                                 ***//
//***    14.06.2003: Changed Namespace from                        ***//
//***                SharpPrivacy.OpenPGP to                       ***//
//***                SharpPrivacy.SharpPrivacyLib.OpenPGP          ***//
//***    31.10.2013  James R. Stembridge - Changed Namespace to    ***//
//***                Hermes.Crypto.OpenPgp                         ***//
//***                Changed so that it works in WinRT.            ***//
//***                Changed to handle the LR/CF format problems   ***//
//***                found when using Windows 8 Mail.              ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;

namespace Hermes.Crypto.OpenPGP {
	
	public class Armor {

        private static string signatureHeader;
        private static string signatureArmored;
        private static string signatureMessage;

        public static string SignatureHeader
        {
            get
            {
                return signatureHeader;
            }
        }

        public static string SignatureArmored 
         {
            get
            {
                return signatureArmored;
            }
        }

        public static string SignatureMessage
        {
            get
            {
                return signatureMessage;
            }
        }

		public static string RemoveClearSignatureArmor(string strMessage, ref ArmorTypes atType, ref string strSignature) {
			string[] strLines = strMessage.Split('\n');
            if (strLines.Length == 1)
                strLines = strMessage.Split('\r');
			string strReturn = "";
			
			/* Codes for Positions:
			 *  0: Not in an OpenPGP Message
			 *  1: Armor Headers
			 *  2: OpenPGP Message
			 *  3: PGP Signature
			 */
			int iPosition = 0;
			
			for (int i=0; i<strLines.Length; i++) {
                if ((strLines[i].Length > 10) && (strLines[i].Trim().Substring(0, 2) == "--") && (iPosition == 2))
                {
					strLines[i] = strLines[i].Trim();
					iPosition = 0;
					switch (strLines[i]) {
						case "-----BEGIN PGP SIGNATURE-----":
							string[] strRestLines = new string[strLines.Length - i];
							Array.Copy(strLines, i, strRestLines, 0, strLines.Length - (i+1));
							string strRest = String.Join("\n", strRestLines);
							strSignature = Armor.RemoveArmor(strRest, ref atType, ref strRest);
							return strReturn.Substring(0, strReturn.Length - 2);
							
						default:
							iPosition = 2;
							break;
					}
				}
				
				if (iPosition == 2) {
					strReturn += strLines[i].TrimEnd(null) + "\r\n";
				}
				
				if ((iPosition == 1) && (strLines[i].Trim().Length == 0)) {
					iPosition = 2;
				}
				
				if ((strLines[i].Length > 10) && (strLines[i].Trim().Substring(0, 2) == "--") && (iPosition == 0)) {
					strLines[i] = strLines[i].Trim();
					iPosition = 1;
					switch (strLines[i]) {
						case "-----BEGIN PGP SIGNED MESSAGE-----":
							atType = ArmorTypes.OpenPGPSignature;
							break;
						
						default:
							iPosition = 0;
							break;
					}
				}
			}
			
			return "";
		}
		
		public static string RemoveArmor(string strMessage, ref ArmorTypes atType, ref string strRest) {
			string[] strLines = strMessage.Split('\n');
            if (strLines.Length == 1)
                strLines = strMessage.Split('\r');
			string strReturn = "";
			bool foundSignedMessage = false;
			
			/* Codes for Positions:
			 *  0: Not in an OpenPGP Message
			 *  1: Armor Headers
			 *  2: OpenPGP Message
			 */
			int iPosition = 0;

            for (int i = 0; i < strLines.Length; i++)
            {
                if (strLines[i].Length > 10)
                {
                    string wkStr = strLines[i].Trim().Substring(0, 2);
                }
                if ((strLines[i].Length > 10) && (strLines[i].Trim().Substring(0, 2) == "--") && (iPosition == 2))
                {
                    strLines[i] = strLines[i].Trim();
                    iPosition = 0;
                    switch (strLines[i])
                    {
                        case "-----END PGP MESSAGE-----":
                        case "-----END PGP PUBLIC KEY BLOCK-----":
                        case "-----END PGP PRIVATE KEY BLOCK-----":
                        case "-----END PGP SIGNATURE-----":
                            string[] strRestLines = new string[strLines.Length - (i + 1)];
                            Array.Copy(strLines, i + 1, strRestLines, 0, strLines.Length - (i + 1));
                            strRest = String.Join("\n", strRestLines);
                            return strReturn;

                        default:
                            iPosition = 2;
                            break;
                    }
                }


                int isComment = strLines[i].IndexOf(": ");

                //if (isComment != -1)
                if ((iPosition == 1) && (isComment == -1))
                {
                    iPosition = 2;
                }

                if ((iPosition == 2) && (isComment == -1))
                {
                    //int isComment = strLines[i].IndexOf(": ");
                    //if (isComment == -1)
                        strReturn += strLines[i].Trim();
                }

                

                if ((iPosition == 1) && (strLines[i].Trim().Length == 0))
                {
                    iPosition = 2;
                }

                if ((strLines[i].Length > 10) && (strLines[i].Trim().Substring(0, 2) == "--") && (iPosition == 0))
                {
                    strLines[i] = strLines[i].Trim();
                    iPosition = 1;
                    switch (strLines[i])
                    {
                        case "-----BEGIN PGP MESSAGE-----":
                            atType = ArmorTypes.OpenPGPMessage;
                            break;

                        case "-----BEGIN PGP PUBLIC KEY BLOCK-----":
                            atType = ArmorTypes.PublicKeyBlock;
                            break;

                        case "-----BEGIN PGP PRIVATE KEY BLOCK-----":
                            atType = ArmorTypes.PrivateKeyBlock;
                            break;

                        case "-----BEGIN PGP SIGNATURE-----":
                            if (!foundSignedMessage)
                                atType = ArmorTypes.OpenPGPSignature;
                            break;

                        case "-----BEGIN PGP SIGNED MESSAGE-----":
                            atType = ArmorTypes.OpenPGPSignedMessage;
                            iPosition = 0;
                            foundSignedMessage = true;
                            break;

                        default:
                            iPosition = 0;
                            break;
                    }
                }
            }
			
			return "";
			
		}
		
		/// <summary>
		/// Armors an Radix64 encoded secret key.
		/// </summary>
		/// <param name="strKey">The secret key encoded in radix64.</param>
		/// <returns>Returns the armored secret key.</returns>
		public static string WrapPrivateKey(string strKey) {
			string strReturn = "-----BEGIN PGP PRIVATE KEY BLOCK-----\r\n";
			//strReturn += "Version: " + SharpPrivacyLibrary.ApplicationVersionInfos + "\r\n\r\n";
            strReturn += "Version: MyPCP v1.0\r\n\r\n";
			strReturn += strKey;
			strReturn += "-----END PGP PRIVATE KEY BLOCK-----\r\n\r\n";
			
			return strReturn;
		}
		
		/// <summary>
		/// Armors an OpenPGP encoded secret key.
		/// </summary>
		/// <param name="bKey">The secret key encoded in OpenPGP format.</param>
		/// <returns>Returns the armored secret key.</returns>
		public static string WrapPrivateKey(byte[] bKey) {
			return WrapPrivateKey(Radix64.Encode(bKey, true));
		}
		
		/// <summary>
		/// Armors an Radix64 encoded public key.
		/// </summary>
		/// <param name="strKey">The public key encoded in radix64.</param>
		/// <returns>Returns the armored public key.</returns>
		public static string WrapPublicKey(string strKey) {
			string strReturn = "-----BEGIN PGP PUBLIC KEY BLOCK-----\r\n";
			//strReturn += "Version: " + SharpPrivacyLibrary.ApplicationVersionInfos + "\r\n\r\n";
            strReturn += "Version: MyPCP v1.0\r\n\r\n";
			strReturn += strKey;
			strReturn += "-----END PGP PUBLIC KEY BLOCK-----\r\n\r\n";
			
			return strReturn;
		}
		
		/// <summary>
		/// Armors an OpenPGP encoded public key.
		/// </summary>
		/// <param name="bKey">The public key encoded in OpenPGP format.</param>
		/// <returns>Returns the armored public key.</returns>
		public static string WrapPublicKey(byte[] bKey) {
			return WrapPublicKey(Radix64.Encode(bKey, true));
		}

		/// <summary>
		/// Armors an Radix64 encoded OpenPGP message.
		/// </summary>
		/// <param name="strKey">The OpenPGP message encoded in radix64.</param>
		/// <returns>Returns the armored OpenPGP message.</returns>
		public static string WrapMessage(string strMessage) {
			string strReturn = "-----BEGIN PGP MESSAGE-----\r\n";
            //strReturn += "Charset: ISO-8859-1\r\n";
			//strReturn += "Version: " + SharpPrivacyLibrary.ApplicationVersionInfos + "\r\n\r\n";
            strReturn += "Version: MyPCP v1.0\r\n\r\n";
			strReturn += strMessage;
			strReturn += "-----END PGP MESSAGE-----\r\n";
			
			return strReturn;
		}
		
		/// <summary>
		/// Armors a binary formated OpenPGP message.
		/// </summary>
		/// <param name="bKey">The OpenPGP message in binary form.</param>
		/// <returns>Returns the armored OpenPGP message.</returns>
		public static string WrapMessage(byte[] bMessage) {
			return WrapMessage(Radix64.Encode(bMessage, true));
		}
		
		/// <summary>
		/// Armors an OpenPGP Cleartextsignature.
		/// </summary>
		/// <param name="strMessage">The Message that has been signed. Note that it
		/// must not be Dash Escaped before handing it to this function.</param>
		/// <param name="strSignature">The OpenPGP signature of the given 
		/// message formated in Radix64.</param>
		/// <returns>The armored OpenPGP cleartext signature.</returns>
		public static string WrapCleartextSignature(string strMessage, string strSignature) {

            signatureHeader = "-----BEGIN PGP SIGNED MESSAGE-----\r\n";
            signatureHeader += "Hash: SHA1\r\n\r\n";
            signatureMessage = Radix64.DashEscape(strMessage);
            signatureArmored = "\r\n-----BEGIN PGP SIGNATURE-----\r\n";
            signatureArmored += "Version: MyPCP v1.0\r\n\r\n";
            signatureArmored += strSignature;
            signatureArmored += "-----END PGP SIGNATURE-----\r\n";	

			string strFinal = "-----BEGIN PGP SIGNED MESSAGE-----\r\n";
			strFinal += "Hash: SHA1\r\n\r\n";
			strFinal += Radix64.DashEscape(strMessage);
			strFinal += "\r\n-----BEGIN PGP SIGNATURE-----\r\n";
			//strFinal += "Version: " + SharpPrivacyLibrary.ApplicationVersionInfos + "\r\n\r\n";
            strFinal += "Version: MyPCP v1.0\r\n\r\n";
			strFinal += strSignature;
			strFinal += "-----END PGP SIGNATURE-----\r\n";	
			
			return strFinal;
		}

		/// <summary>
		/// Armors an OpenPGP Cleartextsignature.
		/// </summary>
		/// <param name="strSignature">The OpenPGP signature of the given 
		/// message formated in Radix64.</param>
		/// <returns>The armored OpenPGP cleartext signature.</returns>
		public static string WrapCleartextSignature(string strSignature) {
			string strFinal = "\r\n-----BEGIN PGP SIGNATURE-----\r\n";
			//strFinal += "Version: " + SharpPrivacyLibrary.ApplicationVersionInfos + "\r\n\r\n";
            strFinal += "Version: MyPCP v1.0\r\n\r\n";
			strFinal += strSignature;
			strFinal += "-----END PGP SIGNATURE-----\r\n";	
			
			return strFinal;
		}
		
		/// <summary>
		/// Armors an OpenPGP Cleartextsignature.
		/// </summary>
		/// <param name="strMessage">The Message that has been signed. Note that it
		/// must not be Dash Escaped before handing it to this function.</param>
		/// <param name="bSignature">The OpenPGP signature of the given 
		/// message formated in binary.</param>
		/// <returns>The armored OpenPGP cleartext signature.</returns>
		public static string WrapCleatextSignature(string strMessage, byte[] bSignature) {
			return WrapCleartextSignature(strMessage, Radix64.Encode(bSignature, true));
		}
		
		
	}
}
