﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** CipherHelper.cs:                                             ***//
//*** 	This class provides various static helper functions for    ***//
//***   work with ciphers.                                         ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    Version: 0.1.0 (initial release)                          ***//
//***    04.04.2003: Daniel Fabian - Created this file.            ***//
//***    01.06.2003: Daniel Fabian - Added header for the first    ***//
//***                beta release.                                 ***//
//***    14.06.2003: Changed Namespace from                        ***//
//***                SharpPrivacy.OpenPGP to                       ***//
//***                SharpPrivacy.SharpPrivacyLib.OpenPGP          ***//
//***    31.10.2013  James R. Stembridge - Changed Namespace to    ***//
//***                Hermes.Crypto.OpenPgp                         ***//
//***                Changed so that it works in WinRT.            ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;
using Hermes.Crypto.Cipher;

namespace Hermes.Crypto.OpenPGP
{

    public class CipherHelper
    {

        public static int CipherBlockSize(SymAlgorithms saAlgo)
        {
            int iBlockSize = 0;
            switch (saAlgo)
            {
                case SymAlgorithms.AES128:
                case SymAlgorithms.AES192:
                case SymAlgorithms.AES256:
                    iBlockSize = 16;
                    break;
                case SymAlgorithms.CAST5:
                    iBlockSize = 8;
                    break;
                case SymAlgorithms.Triple_DES:
                    iBlockSize = 8;
                    break;
            }
            return iBlockSize;
        }

        public static int CipherKeySize(SymAlgorithms saAlgo)
        {
            int iKeySize = 0;
            switch (saAlgo)
            {
                case SymAlgorithms.AES128:
                    iKeySize = 128;
                    break;
                case SymAlgorithms.AES192:
                    iKeySize = 192;
                    break;
                case SymAlgorithms.AES256:
                    iKeySize = 256;
                    break;
                case SymAlgorithms.CAST5:
                    iKeySize = 128;
                    break;
                case SymAlgorithms.Triple_DES:
                    iKeySize = 192;
                    break;
            }
            return iKeySize;
        }

        public static SymmetricAlgorithm CreateSymAlgorithm(SymAlgorithms saAlgo)
        {
            SymmetricAlgorithm saReturn;
            var messageLoader = new Windows.ApplicationModel.Resources.ResourceLoader();
            switch (saAlgo)
            {
                case SymAlgorithms.AES128:
                    saReturn = Rijndael.Create();
                    saReturn.BlockSize = 128;
                    saReturn.KeySize = 128;
                    break;
                case SymAlgorithms.AES192:
                    saReturn = Rijndael.Create();
                    saReturn.BlockSize = 128;
                    saReturn.KeySize = 192;
                    break;
                case SymAlgorithms.AES256:
                    saReturn = Rijndael.Create();
                    saReturn.BlockSize = 128;
                    saReturn.KeySize = 256;
                    break;
                case SymAlgorithms.CAST5:
                    saReturn = CAST5.Create();
                    break;
                case SymAlgorithms.Triple_DES:
                    saReturn = TripleDES.Create();
                    break;
                default:
                    //throw new System.Security.Cryptography.CryptographicException("The algorithm is not supported!");
                    throw new Exception(messageLoader.GetString("ErrorUncompressingUnknownAlgo"));
                //throw new Exception(messageLoader.GetString("CipherHelperAlgoNotSupported"));
                //throw new Exception("Cipherhelper: The algorithm is not supported!");
            }

            return saReturn;
        }

    }
}
