﻿//*** ************************************************************ ***//
//*** ************************************************************ ***//
//*** Hermes Crypto Library                                        ***//
//*** ************************************************************ ***//
//***                                                              ***//
//***            Copyright © 2013 James R. Stembridge              ***//
//***                                                              ***//
//*** This file is part of the source code distribution of the     ***//
//*** Hermes Crypto Library.  This work is a derivative work       ***//
//*** based on of the source code distribution of SharpPrivacy.    ***//
//*** SharpPrivacy was written and copyrighted under the Gnu       ***//
//*** General Public License by Daniel Fabian in 2003. The         ***//
//*** original source code implementation of SharpPrivacy was      ***//
//*** found at http://sourceforge.net/projects/sharpprivacy/.      ***//
//*** The SharpPrivacy project seems to be dead now, but the       ***//
//*** source code was still available at the time of this writing. ***//
//***                                                              ***//
//*** The Hermes Crypto Library is an implementation of the        ***//
//*** openPGP standard, as defined by RFC4880, designed to operate ***//
//*** in the WinRT (Metro) environment. The code is designed to    ***//
//*** be compiled as a class library to create a dll that you can  ***//
//*** dynamically invoke from other programs.                      ***//
//***                                                              ***//
//*** This Hermes Crypto Library is free software; you can         ***//
//*** redistribute it and/or modify it under the terms of          ***//
//*** version 2 of the GNU General Public License as published by  ***//
//*** the Free Software Foundation.                                ***//
//***                                                              ***//
//*** This work is distributed in the hope that it will be useful, ***//
//*** but WITHOUT ANY WARRANTY; without even the implied warranty  ***//
//*** of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See  ***//
//*** the GNU General Public License for more details. If you      ***//
//*** cannot comply with this FOR ANY REASON, then you do not have ***//
//*** a license to run, copy, modify, or otherwise use this work.  ***//
//***                                                              ***//
//*** You should have received a copy of the GNU General Public    ***//
//*** License along with this program; if not, write to the        ***//
//*** Free Software Foundation, Inc., 59 Temple Place, Suite 330,  ***//
//*** Boston, MA  02111-1307  USA                                  ***//
//*** or visit their web site at                                   ***//
//*** http://www.gnu.org/copyleft/gpl.html.                        ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Message.cs:                                                  ***//
//*** 	Abstract class that defines what a message should look     ***//
//***   like.                                                      ***//
//***                                                              ***//
//*** ************************************************************ ***//
//***                                                              ***//
//*** Change Log:                                                  ***//
//***    Version: 0.1.0 (initial release)                          ***//
//***    02.03.2003: Daniel Fabian - Created this file.            ***//
//***    01.06.2003: Daniel Fabian - Added header for the first    ***//
//***                beta release.                                 ***//
//***    14.06.2003: Changed Namespace from                        ***//
//***                SharpPrivacy.OpenPGP.Messages to              ***//
//***                SharpPrivacy.SharpPrivacyLib.OpenPGP.Messages ***//
//***    31.10.2013  James R. Stembridge - Changed Namespace to    ***//
//***                Hermes.Crypto.OpenPgp.Messages.               ***//
//***                Changed so that it works in WinRT.            ***//
//***                                                              ***//
//*** ************************************************************ ***//
//*** ************************************************************ ***//
//
using System;
using Hermes.Crypto.Cipher;

namespace Hermes.Crypto.OpenPGP.Messages {

	public abstract class Message : object {
		
		protected Packet[] pPackets;
		
		/// <summary>
		/// Returns an array of packets that form the current
		/// message. Readonly property.
		/// </summary>
		/// <value>An array of packets that form the current
		/// message. Readonly property.</value>
		/// <remarks>No remarks</remarks>
		public Packet[] Packets {
			get {
				return pPackets;
			}
		}
		
		/// <summary>
		/// Parses an array of packets into the message
		/// </summary>
		/// <param name="packets">Array of packet that forms the 
		/// message</param>
		/// <returns>Returns the number of packets parsed.</returns>
		/// <remarks>No remarks</remarks>
		public abstract int ParseMessage(Packet[] packets);
		
		/// <summary>
		/// Returns the message in an RFC2440 encoded way
		/// </summary>
		/// <returns>Returns a byte representation of the encoded
		/// message</returns>
		/// <remarks>No remarks</remarks>
		public abstract byte[] GetEncoded();
		
		/// <summary>
		/// Returns a byte array that contains the message in
		/// a form that a hash for signing can be done over the 
		/// message.
		/// </summary>
		/// <returns>a byte array that contains the message in
		/// a form that a hash for signing can be done over the 
		/// message.</returns>
		/// <remarks>No remarks</remarks>
		//public abstract byte[] GetSignatureData();
	}
}
